from nzshm_common.grids.io import latlon_from_base64_zip


class NZ01nb1v1:
    nz = None

    def load(self):
        if self.nz is None:
            self.nz = latlon_from_base64_zip(data)
        return self.nz


data = (
    "UEsDBD8AAgAOADxX81S60g40OQgAAPm9AAAEAAAAZGF0YQkEBQBdAACAAAAY"
    "jaxcNpfygoevll45+3II7cxm7iUwhrlQ97Th69ymR0q/sK97FS9DzGoX6dMx"
    "F3QtTVZkGQ4y2pyR5wPopKJDZGCFRWosJskYY90YAjRx6UCc6DnfRtzdgpFN"
    "Md9WfbMNi0hAPGKv6ZAuYtqStgsEL8OjuLqMvUplWbx01KOTEc++On40nzQ8"
    "hsg5oZcaDQNyzeWNLIrTULom7u743FB5hjgk7Bd/GN6fcINDdbQvopAO/hC9"
    "u1YiEl/KwwAtt8nXYGnhaej6Qdv31fBEQK9X+MZA51xNvovdXCgmqZ+0fo0z"
    "lHrWw01JS+KQJxNm5hyTVHguYTyF9g2866ZTS/z5x2wYHuUOMO4u11Ba6CdV"
    "2n/NTn+ekwGqrXujDWdjDjhx2mZSlMN/pvHqFAazB2JvRBGXDCUbb3mlE61N"
    "3Nf44MB1LA/5GTJf/GHpmQrDl5c7ibVWkDukRASl6jRLfEYbV5lj2Qfr4Zs9"
    "UXC/Hb8xK0MEUkHfixlAsmA2hZwppNXlaTV60ykLkiSrNcJh67ooLgfqPTll"
    "U3egTG124E+FPF1SLs3kR7JO4YNSAzbk951XzFok9ulvILmPPGRD64Q7hTQU"
    "SkdE77OW88R+3Y8tGhmcKXfKM65k6wxcXLntRCVVgpF1eu8amNaXlIXPHS3y"
    "3GoYNjWii1kqM4ohdM+AwUOTOHwxn1tNVBIPhlCfbZ+W7lJ/O5Jq7uZ6aJHh"
    "TTIrO7cKwR29lc6CppA5n5ubQdiMU1O5r9diqlAT6ANgX7tusX5jV0dmZFBh"
    "G+Zu9loM6R8KXu9w4qJ+z8WmzYL18m2qWzPBeDbexYmtOtL0jK88eYv8BY22"
    "0A5uzld9PhV2WFtFQZqHePcNr0dWtNE/thzYsPj+QbPIywXeg0ihkhch7rnp"
    "02RFHx558ex2k0u8O5Hl8mccI3gsranGEPuj6NPf153FdYYyZp/1feGLJ0Bk"
    "WUGeEt0BtSTX8CwW3q0geupvbP7al/tkcGgMLi5+sZlAEVNu5MKtbKwIGFHG"
    "Z5Z60w2qeDpp23hqwRIBxRKKNA5opBkAAjHKI6LDrGVjtmY+jbXT1BYGRSut"
    "mUDInN3t9+e7r47gQ/re2pEbrl65JFZJHG0580cnqxus+nkxd7rAedcjmu3q"
    "fPSwZMNIlzfra1IbAlzm9XyGwB+izEN2E2QJ1gTzwVvBxzWMjVjYUKB460vW"
    "4KY9z7b6BX+H2qneP8iYUWNM5pfLlEtsB76Qj2S8udgGPmAguRS3x4dv7oMI"
    "8/mH/6KHVydaeUF8oYNnwnCsygjJcsdZpvsMMEGgtkiTAJjLpvL4ZBra7A3g"
    "v6aGLkg2uPVkXK7PDET+Vl5/CW04BtwqtosToqMUNNYPJMaALWuKqLGZCMVw"
    "Hy6hRooiRiGZUd/uUbxnIrlXPY0NLY20sw+MVAr39cPU40qMNQh21Ge2du9I"
    "1zU7wTIP2f3cWbNbnyXH+3R+VouMV9OOiaq6ci8zx34Zzufh8WHU5qKzDU6t"
    "yAmmyKw0Hu+jOqOwDYaS+rWqPoPpg7hBXZTsEJ6z/yAFrN2NlymXQ0ko7vIB"
    "QPcxyLJdi+qLybg+JoTCWJICXvVyLYFDYLsTuHVsax4WmI2XtIHNMx62ViwW"
    "eO5Izc9eafxiIdqpsU/HBUSmlSfKOpFHd88B2IXORZ6FnWMURvxQRDvYpxPd"
    "AlwDCSaE4+SDdYFkVkEI8nSUwARhw6QnHiD30dUOap+5EwptZ9VB3MXaA5j8"
    "a5FCCdFMK02Y5JtMa2LQqVBcykBcr0XFzPcXElumKRD8LDDITPBnP3vYt4Y8"
    "AsJBStzQGxk+WJi3VvO/r3EocN6XYcpRJYJ+2oZRJk/HXXEmzKYMzMKLjaK8"
    "mR/EJJTGCFEzabgTBTJANWs9kpK1CF6Yqnhx/dvyYAkuTJfEnfFt2Y0eseHm"
    "e12Dk9t5tBvi68XWkWkD+5jA7C3GINmVvrV4aPiZoBOwjxNcomd32hoc8WdL"
    "AGNWvfyOKiA/oQywA0H1f8km+Vgiqq+GYatXkI67ReL4rrCKmQd8HwtW4O81"
    "ygAAQsonaTSHo6PW4P5WJOBDrIytHDXw7MZmMKpa17tHdseH73V17o7CwowO"
    "Ihg1hEnczMRouwlMaA7dbWxm/VnPT95kXxgDU+L3FbET0F6GMMuLpD+s82LQ"
    "wbK3ylDXv3S71lzsNCAja0xMWGEpdKh1b+l2CoQbALRkCZxgR7t0ms+h/J9w"
    "s3tYRhrgYZNa7NznAv6Ok6DmaRepZau0r7WvWVl5obBZ3NFyAmdmsr5lITlI"
    "qjX+QgbpVvrMs4VcnWR7IYYHjcE8lL9gK3jGG1yF80++HGRywLihfaZySpoM"
    "kugLkcRmrEt8lGoYXONHPbvzJe2RaFtnuuTICM6Uc7SdZNNrgP7H9DKhTBcE"
    "hkIKT5l/FgLhuqvgSTUezf9GHfu+oSKnBZTxO4+qYv8GQ3qT/KUB/VMZ8s48"
    "hTMq7sCEPM4LWCpwx6V/FBFG1lF+/jm9224+RxOksdAFqRnRFq3YeTgXqc+m"
    "x/agLfsopBtHYrZNgK7yc0jJuk99touaXzly21H1hcHujpnrh21b7XBr5iex"
    "E7h7dGh/nm0r/bU9a5x08w8r98PLTeL5yMtwzEUY9NklzpzAG2p84M2hjhMz"
    "YhwbdIgn+lCclTyxKuTgtrn03pe3NZqZdNcrvo6M1bRRmqN1XlDacFzTM96W"
    "qaPaN/WIKM1x+TypmhIK2Td2LBBhZ7D9Pl0kxrn1G7ctmNTsQwbl4uC/tu7N"
    "flRQdE4Ii44DI/NEbETngmjBv/9xWDEAUEsBAj8DPwACAA4APFfzVLrSDjQ5"
    "CAAA+b0AAAQAAAAAAAAAAAAAAIABAAAAAGRhdGFQSwUGAAAAAAEAAQAyAAAA"
    "WwgAAAAA"
)
